/*  GUI frontend for common compresion programs
 *  Copyright (C) 2000-2001 Lucian Langa
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <dirent.h>
#include <errno.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <gnome.h>
#include <ctype.h>

#include "misc.h"
#include "conf.h"
#include "components.h"
#include "gnochive.h"
#include "support.h"

extern gchivemain *gnochivemain;

int endpos;

static gboolean check_components_match (const gchar *base,
					const gchar *path,
					gint	     root_pos,
					gint	     len);
					

/* This turns a relative pathname into an absolute one based on the given
   base directory (which MUST be absolute).
   e.g. "/home/damon" + "../dave/test" -> "/home/dave/test"
   The returned path should be freed when no longer needed. */
gchar*
make_absolute_path (const gchar *dir, const gchar *file)
{
  gint dir_pos, file_pos, len, root_pos = 0;
  gchar *path;

  g_return_val_if_fail (dir != NULL, NULL);

  if (file == NULL || file[0] == '\0')
    return g_strdup (dir);

  if (g_path_is_absolute (file))
    return g_strdup (file);

  /* Start at last character in dir. */
  dir_pos = strlen (dir) - 1;

  /* First we make sure we skip any '/' at the end of dir. */
  if (dir_pos > root_pos && dir[dir_pos] == G_DIR_SEPARATOR)
    dir_pos--;

  /* Now for each '..' in file, we step back one component in dir, and
     forward one component in file. */
  file_pos = 0;
  for (;;)
    {
      /* Skip './' */
      if (file[file_pos] == '.' && file[file_pos + 1] == G_DIR_SEPARATOR)
	file_pos += 2;

      else if (file[file_pos] == '.' && file[file_pos + 1] == '.'
	       && (file[file_pos + 2] == G_DIR_SEPARATOR
		   || file[file_pos + 2] == '\0'))
	{
	  while (dir_pos > root_pos && dir[dir_pos] != G_DIR_SEPARATOR)
	    dir_pos--;
	  if (dir_pos > root_pos)
	    dir_pos--;

	  if (file[file_pos + 2] == G_DIR_SEPARATOR)
	    file_pos += 3;
	  else
	    file_pos += 2;
	}

      else
	break;
    }

  /* Now concatenate the parts of dir and file together. */
  if (dir_pos > root_pos)
    dir_pos++;
  len = dir_pos + 1 + (strlen (file) - file_pos) + 1;
  path = g_malloc (len); 
  strncpy (path, dir, dir_pos);
  path[dir_pos] = G_DIR_SEPARATOR;
  strcpy (path + dir_pos + 1, file + file_pos);
  return path;
}


/* This turns an absolute pathname into an relative one based on the given
   base directory. Both arguments must be absolute paths, and should be in
   canonical form, i.e. not containing '..', '.' or multiple '/'s together.
   The returned value may or may not end with a '/', depending on the
   arguments.
   e.g. "/home/damon" + "/home/damon/pixmaps" -> "pixmaps"
        "/home/damon" + "/home/damon/pixmaps/" -> "pixmaps/"
        "/home/damon/project" + "/home/damon/test/pic.xpm" -> "../test/pic.xpm"
        "/home/damon/project" + "/home/damon/project" -> ""
        "/home/damon/project" + "/home/damon" -> "../"
   The returned path should be freed when no longer needed. */
gchar*
make_relative_path (const gchar *base, const gchar *path)
{
  gchar *relative_path;
  gint pos, num_parents = 0, len, path_len, i, root_pos = 0;
  gboolean match = FALSE;

  if (path == NULL)
    return g_strdup ("");

  /* We step up each component of base_dir until we find a match with the
     start of file. */
  pos = strlen (base) - 1;
  for (;;)
    {
      /* Skip trailing '/'s. */
      if (pos > root_pos && base[pos] == G_DIR_SEPARATOR)
	pos--;

      match = check_components_match (base, path, root_pos, pos + 1);
      if (match)
	break;

      /* They didn't match, so we move up one component and try again. */
      num_parents++;
      while (pos && base[pos] != G_DIR_SEPARATOR)
	pos--;
    }

  /* If we match some components, build the relative path, else just return
     path. */
  if (match)
    {
      path_len = strlen (path);
      /* Skip over the '/', but special case for root directory. */
      if (pos == root_pos)
	pos++;
      else
	pos += 2;
      len = (num_parents * 3) + 1;
      if (path_len > pos)
	len += path_len - pos;
      relative_path = g_malloc (len);
      /* Add a '../' for each parent directory needed. */
      for (i = 0; i < num_parents; i++)
	{
	  relative_path[i * 3]     = '.';
	  relative_path[i * 3 + 1] = '.';
	  relative_path[i * 3 + 2] = G_DIR_SEPARATOR;
	}
      /* Add on the end of path, skipping the '/' after the component. */
      if (path_len > pos)
	strcpy (relative_path + num_parents * 3, path + pos);
      relative_path[len - 1] = '\0';

      return relative_path;
    }
  else
    return g_strdup (path);
}


static gboolean check_components_match (const gchar *base,
					const gchar *path,
					gint	    root_pos,
					gint	    len)
{
if (strncmp(base,path,len)) return FALSE;
if (len==root_pos+1||path[len]=='\0'||path[len]==G_DIR_SEPARATOR) return TRUE;
return FALSE;
}

/* This returns the parent directory of the given directory, which may or may
   not end in a G_DIR_SEPARATOR. If there is no parent directory, NULL is
   returned. The returned string should be freed. dir MUST be absolute. */
gchar*
get_parent_directory	(const gchar	    *dir)
{
  gchar *skipped_root, *parent_dir;
  gint pos;

  g_return_val_if_fail (g_path_is_absolute (dir), NULL);

  /* We handle the root dir specially here, and return NULL. */
  skipped_root = g_path_skip_root ((gchar*) dir);
  if (*skipped_root == '\0')
    return NULL;

  /* Ignore any G_DIR_SEPARATOR at the end of dir (just by skipping the last
     char), and step back to the previous G_DIR_SEPARATOR. */
  pos = strlen (dir) - 2;
  while (pos >= 0 && dir[pos] != G_DIR_SEPARATOR)
    pos--;

  /* This shouldn't really happen, since we dealt with the root dir above,
     but just in case. */
  if (pos < 0)
    return NULL;

  /* Check if the parent directory is the root directory. If it is, we just
     want to return the root directory, i.e. "/" or "C:\". */
  if (pos <= skipped_root - dir)
    pos = skipped_root - dir;

  parent_dir = g_malloc (pos + 1);
  strncpy (parent_dir, dir, pos);
  parent_dir[pos] = '\0';

  return parent_dir;
}

/* Extracts file name from a specified path
   Path MUST be absolute */
gchar*
extract_filename	(const gchar *fullpath)
{
    gint mlen,i,j;
    gint sep=0;
    gchar *fname;
mlen=strlen(fullpath);
for (i=0;i<=mlen;i++)
{
if (fullpath[i]==G_DIR_SEPARATOR) sep++;
}
if (sep==0) {
	    fname=g_malloc(strlen(fullpath));
	    memcpy(fname,fullpath,strlen(fullpath));
	    return fname;
	    }
i=j=1;
while (j<sep) {
 while (fullpath[i]!=G_DIR_SEPARATOR) i++;
i++;j++;    
}
fname=g_malloc(mlen-i+1);
memcpy(fname,fullpath+i,mlen-i+1);
return fname;
}

/*
 * returns the type of file for a given [path+]file
 * if a path is providen then it returns null
 */
 
int
f_type				(const gchar *path)
{
if (path[strlen(path)-1]==G_DIR_SEPARATOR) g_print("path\n");
//g_print("%s",extract_filename(path));
return 0;
}

/* null if line is not a dir
 * else root level of the line
 */

int
sort_dirs			(gchar *line)
{
	if (line[strlen(line)-1] == G_DIR_SEPARATOR) 
		return 1;
	else return 0;
}

gboolean
sort_type		(gchar *line, gchar *attr)
{
	if ((line[strlen(line)-1] == G_DIR_SEPARATOR) || (attr[0] == 'd'))
		return 1;
	else return 0;
}

int
sort_subdirs			(gchar *parent, gchar *child)
{
	int match=0;
	int i=0;
	int j;
	j=strlen(parent);
	while (j!=0)
	{
		if (parent[i]==child[i]) {
			i++;
			j--;
			match=1;
		}
		else {
			j--;
			i++;
			match=0;
			}
	}
	if (match) {
		//check if we really found a child
		if (child[i++]!=0) return 1;
		else return 0;
		}
	else return 0;
}

/* returns nuber of separators */
int
process_line			(gchar *line)
{
int i=0;
int seps=0;
if (line[strlen(line)-1]==G_DIR_SEPARATOR)
 	seps=1;
//else {
		for (i=0;i<strlen(line);i++)
			if (line[i]==G_DIR_SEPARATOR) seps++;
	//}
return seps;
}

// sep make root work of sep

gchar*
extract_root_word		(gchar *line, int sep)
{
	gchar tmp[0x100];
	int i=0;
	int isep=0;
	int j=0;
	
	while (isep<sep) {
		while (line[i] != G_DIR_SEPARATOR && i<strlen(line)) {
				tmp[j]=line[i];
				i++;j++;
		}
		tmp[j] = G_DIR_SEPARATOR;
		i++;j++;
		isep++;
	}

	//tmp[j++]=G_DIR_SEPARATOR;
	tmp[j++]='\0';
	return g_strdup(tmp);
}

gchar*
trim_root_word			(gchar *file1,gchar *file2)
{
gchar tmp[0x100];
gchar tmp1[0x100];
strcpy(tmp,"/");
strcat(tmp,file1);
strcpy(tmp1,"/");
strcat(tmp1,file2);
return make_relative_path(tmp,tmp1);
}

/*
 *	tar.gz -> tar.bz2
 */

gchar*
convert_arch_name		(gchar *arch,gchar *newpref)
{
	int i=0;
	int j=0;
	gchar temp[0x100];
	gchar *tmp;
	
	while (arch[i]!='.')
	{
		temp[j]=arch[i];
		i++;
		j++;
	}
	tmp=malloc(strlen(temp));
	strcpy(tmp,temp);
	return tmp;
}

/*
    extracts the extension of a given file_name
    (a quick dirty patch for excluding tar files to be tested)
 */
/*gchar*
extract_extension		(const gchar *file_name)
{
gchar tmp[0x100];
gchar ext[0x10]='.tar';
guint i=0,j;
strcpy(tmp,file_name);
while (tmp[i]!='.') i++;
while (tmp[i]!='\0')
 {
 while (j<=strlen(ext)) {
    if (tmp[i]==ext[j])
	    {
	    i++;j++;
	    }
    i++;
 }
 }
}
}*/


guint
pos_compare			(const gchar *s1,const gchar *s2)
{
int pos1=0,pos2=0;
int mycmp=-1;
int vfon=0;
while (s1[pos1]!='\0') {
	if (s1[pos1]==s2[pos2]) {pos1++;pos2++;vfon=1;}
	else {if (vfon==0) {
				pos1++;
				pos2=0;
				}
		if (vfon) {
				pos1++;pos2++;
				vfon=0;
				}
		}
	if (s2[pos2]=='\0') {endpos=pos1;mycmp=0;}
	}
return mycmp;
}

/*FIXME*/
gboolean
gnochive_check_file(gchar *file_name,guint mode)
{
	switch (mode) {
		case 0: return is_readable(file_name);break;
		case 1: return is_writable(file_name);break;
		case 2:
		{
			if (!is_readable(file_name))
				return FALSE;
			if (!is_writable(file_name))
				return FALSE;
		}
	}
	return TRUE;
}

gboolean
is_dir (const gchar *file)
{
	struct stat sts;
	int res;
	if (!file) return FALSE;
	res=stat(file,&sts);
	if (res) return FALSE;
		if (S_ISDIR(sts.st_mode)) return TRUE;
	return FALSE;
}

gboolean
is_regular (const gchar *file)
{
	struct stat sts;
	int res;
	if (!file) return FALSE;
	res=stat(file,&sts);
	if (res) return FALSE;
		if (S_ISREG(sts.st_mode)) return TRUE;
	return FALSE;
}

gboolean
is_readable (const gchar *file)
{
	struct stat sts;
	int res;
	
	if (!file)
		return FALSE;
	res = stat(file, &sts);
	if (res)
		return FALSE;
	if (S_IRUSR & sts.st_mode)
		return TRUE;
	return FALSE;
}

gboolean
is_writable (const gchar * fn)
{
	struct stat st;
	int ret;
	if (!fn)
		return FALSE;
	ret = stat (fn, &st);
	if (ret)
		return FALSE;
	if (S_IWUSR & st.st_mode)
		return TRUE;
	return FALSE;
}

gboolean
gnochive_can_create(const gchar *file)
{
	FILE *fn;
	fn=fopen(file,"w");
	
	if (fn)
	{
		fclose(fn);
		remove(file);
		return TRUE;
	}
	else return FALSE;
}

void
open_xpm 			(char *filename,
				GtkWidget *parent, GdkPixmap **pixmap,
				GdkBitmap **mask, int quit_on_err)
{
   	gchar *temp1str;
	gchar *op_err;
   GtkStyle *style;
//   GdkBitmak *mask;

   if (*filename == '\0') return;
   style = gtk_widget_get_style (parent);
   temp1str = g_strconcat (BASE_CONF_DIR, "/", filename, NULL);
   /*expand_path (temp1str, tempstr, sizeof (tempstr));
   if (access (tempstr, F_OK) != 0) {
      g_free (temp1str);
      temp1str = g_strconcat (SHARE_DIR, "/", filename, NULL);
      //expand_path (temp1str, tempstr, sizeof (tempstr));
      if (access (tempstr, F_OK) != 0) {
         g_free (temp1str);
         temp1str = g_strconcat ("/usr/share/icons/", filename, NULL);
         //expand_path (temp1str, tempstr, sizeof (tempstr));
         if (access (tempstr, F_OK) != 0) {
            g_free (temp1str);
            if (!quit_on_err) return;
            printf ("Error: Cannot find file %s in %s or %s\n", filename, SHARE_DIR, BASE_CONF_DIR);
            exit (-1);
         }
      }
   }*/
   *pixmap = gdk_pixmap_create_from_xpm (parent->window, mask, &style->bg[GTK_STATE_NORMAL], temp1str);
   if (*pixmap == NULL) {
      if (!quit_on_err) return;
	  op_err=g_strconcat(_("Error: Error opening file: "),temp1str,NULL);
	  g_error(op_err);
      exit (-1);
   }
   g_free (temp1str);
}

void
xfer_mode(mode_t my,gchar *mode)
{
	(void)strcpy(mode,"----------");
if(my & (S_IEXEC>>6)) mode[9]='x';
if(my & (S_IWRITE>>6)) mode[8]='w';
if(my & (S_IREAD>>6)) mode[7]='r';
if(my & (S_IEXEC>>3)) mode[6]='x';
if(my & (S_IWRITE>>3)) mode[5]='w';
if(my & (S_IREAD>>3)) mode[4]='r';
if(my & S_IEXEC) mode[3]='x';
if(my & S_IWRITE) mode[2]='w';
if(my & S_IREAD) mode[1]='r';
if(my & S_ISVTX) mode[9]='t';		/* Sticky bit */
if(my & S_ISGID) mode[6]='s';		/* Set grup id */
if(my & S_ISUID) mode[3]='s';		/* Set user id */
switch(my & S_IFMT){
	case S_IFDIR:mode[0]='d';break;
	case S_IFCHR:mode[0]='c';break;
	case S_IFBLK:mode[0]='b';break;
	case S_IFLNK:mode[0]='l';break;
	case S_IFREG:mode[0]='-';break;
	default	    :mode[0]='?';
}
}

/* gnochive_istr_has_suffix
 * copy-pasted from Nautilus
 */
gboolean
gnochive_istr_has_suffix (const char *haystack, const char *needle)
{
	const char *h, *n;
	char hc, nc;

	if (needle == NULL) {
		return TRUE;
	}
	if (haystack == NULL) {
		return needle[0] == '\0';
	}
		
	/* Eat one character at a time. */
	h = haystack + strlen (haystack);
	n = needle + strlen (needle);
	do {
		if (n == needle) {
			return TRUE;
		}
		if (h == haystack) {
			return FALSE;
		}
		hc = *--h;
		nc = *--n;
		hc = tolower ((guchar) hc);
		nc = tolower ((guchar) nc);
	} while (hc == nc);
	return FALSE;
}

gint
gnochive_exec_arch	(gchar *path, gchar *param, ...)
{
	gchar *mymsg;
	gchar *temp;
	gchar *arg_list[20];
	guint i = 1, j;
	pid_t pid;
	va_list arg;
	arg_list[0] = g_strdup(param);
	va_start(arg, param);
	while ((mymsg = va_arg(arg, gchar *)) != NULL)
	{
		arg_list[i] = g_strdup(mymsg);
		i++;
	}
	va_end(arg);
	arg_list[i] = NULL;
	
	//for (j = 0;j<=i;j++)
		//g_print("par:%s\n", arg_list[j]);
	
	if (check_arc(path))
	{
		if((pid = fork())==0)
			execv(path, arg_list);
		for (j = 0;j<i;j++)
			g_free(arg_list[j]);
		return pid;
	}
	else
	{
		temp=g_strdup_printf(_("%s was not found! No action taken!"),arg_list[0]);
		gnome_app_flash(GNOME_APP(gnochivemain->mainwin),temp);
		for (j = 0;j<i;j++)
			g_free(arg_list[j]);
		g_free(temp);
		return FALSE;
	}
}

gchar*
gnochive_remove_ext (archivedesc *ad)
{
	gchar *new, *new1;
	
	if (!cmpare(ad->type, "tgz"))
	{
		new = g_strndup(ad->name, strlen(ad->name) - strlen(strstr(ad->name, ".tar.gz")));
		new1 = g_strconcat(new, "\0", NULL);
		g_free(new);
		return new1;
	}
	if (!cmpare(ad->type, "tbz2"))
	{
		new = g_strndup(ad->name, strlen(ad->name) - strlen(strstr(ad->name, ".tar.bz2")));
		new1 = g_strconcat(new, "\0", NULL);
		g_free(new);
		return new1;
	}
	if (!cmpare(ad->type, "comp"))
	{
		new = g_strndup(ad->name, strlen(ad->name) - strlen(strstr(ad->name, ".tar.Z")));
		new1 = g_strconcat(new, "\0", NULL);
		g_free(new);
		return new1;
	}
/*		strstr(
		return g_strconcat(file_name,".","rar",NULL);
	if (!cmpare(arc_type,"gz"))
		return g_strconcat(file_name,".","gz",NULL);
	if (!cmpare(arc_type,"bz"))
		return g_strconcat(file_name,".","bz",NULL);
	if (!cmpare(arc_type,"tar"))
	{
		switch (sub_type) {
				case 0:	return g_strconcat(file_name,".","tar",NULL);
				case 1: return g_strconcat(file_name,".","tar",".gz",NULL);
				case 2:	return g_strconcat(file_name,".","tar",".bz2",NULL);
				case 3:	return g_strconcat(file_name,".","tar",".Z",NULL);
				}
			}
	if (!cmpare(arc_type,"zip"))
		return g_strconcat(file_name,".","zip",NULL);
	if (!cmpare(arc_type,"lha"))
		return g_strconcat(file_name,".","lha",NULL);
	if (!cmpare(arc_type,"zoo"))
		return g_strconcat(file_name,".","zoo",NULL);*/
	/*we hopefuly won't get here*/
	return 0;
}

void
glog_print(gchar *msg)
{
	GtkWidget *text = NULL;
	
	text = lookup_widget(GTK_WIDGET(gnochivemain->log_window), "text7");
	gtk_text_freeze(GTK_TEXT(text));
	gtk_text_insert(GTK_TEXT(text), NULL,
			&text->style->black, NULL, msg, strlen(msg));
	gtk_text_thaw(GTK_TEXT(text));
}
